from watchdog.events import PatternMatchingEventHandler
import time


class AnyFileChangedHandler(PatternMatchingEventHandler):
    def __init__(self, callback_registry, patterns=None, *args):
        super().__init__(patterns)
        self._callback_registry = callback_registry
        self._args = args
        self._last_modified_timestamp = None
        self._event_merge_interval = 2

    def on_created(self, event):
        if not event.is_directory and self._is_not_temp_path(event.src_path):
            super().on_created(event)
            self._call_file_changed_callback()

    @staticmethod
    def _is_not_temp_path(path):
        return path.find('~') == -1

    def on_deleted(self, event):
        if not event.is_directory and self._is_not_temp_path(event.src_path):
            super().on_deleted(event)
            self._call_file_changed_callback()

    def on_modified(self, event):
        if not event.is_directory and self._is_not_temp_path(event.src_path):
            now = time.time()
            if self._should_trigger_modified_event(now):
                super().on_modified(event)
                self._last_modified_timestamp = now
                self._call_file_changed_callback()

    def _should_trigger_modified_event(self, now):
        return self._last_modified_timestamp is None \
               or (now - self._last_modified_timestamp) >= self._event_merge_interval

    def _call_file_changed_callback(self):
        self._callback_registry.call()
