# coding: utf-8

"""
    Model representing the identification for an asset.
"""


import pprint


class AssetIdentification:
    """
    Model representing the identification for an asset.
    """
    property_types = {
        'model_name': 'str',
        'model_number': 'int',
        'serial_number': 'str',
        'vendor_name': 'str',
        'vendor_number': 'int'
    }

    serialization_keys = {
        'model_name': 'modelName',
        'model_number': 'modelNumber',
        'serial_number': 'serialNumber',
        'vendor_name': 'vendorName',
        'vendor_number': 'vendorNumber'
    }

    def __init__(self,
                 model_name=None,
                 model_number=None,
                 serial_number=None,
                 vendor_name=None,
                 vendor_number=None):
        self._model_name = None
        self._model_number = None
        self._serial_number = None
        self._vendor_name = None
        self._vendor_number = None

        if model_name is not None:
            self.model_name = model_name
        if model_number is not None:
            self.model_number = model_number
        if serial_number is not None:
            self.serial_number = serial_number
        if vendor_name is not None:
            self.vendor_name = vendor_name
        if vendor_number is not None:
            self.vendor_number = vendor_number

    @property
    def model_name(self):
        """Gets the model_name of this AssetIdentification.

        Model name of the asset.

        :return: The model_name of this AssetIdentification.
        :rtype: str
        """
        return self._model_name

    @model_name.setter
    def model_name(self, model_name):
        """Sets the model_name of this AssetIdentification.

        Model name of the asset.

        :param model_name: The model_name of this AssetIdentification.
        :type: str
        """

        self._model_name = model_name.strip() if model_name is not None else model_name

    @property
    def model_number(self):
        """Gets the model_number of this AssetIdentification.

        Model number of the asset.

        :return: The model_number of this AssetIdentification.
        :rtype: int
        """
        return self._model_number

    @model_number.setter
    def model_number(self, model_number):
        """Sets the model_number of this AssetIdentification.

        Model number of the asset.

        :param model_number: The model_number of this AssetIdentification.
        :type: int
        """

        self._model_number = model_number

    @property
    def serial_number(self):
        """Gets the serial_number of this AssetIdentification.

        Serial number of the asset.

        :return: The serial_number of this AssetIdentification.
        :rtype: str
        """
        return self._serial_number

    @serial_number.setter
    def serial_number(self, serial_number):
        """Sets the serial_number of this AssetIdentification.

        Serial number of the asset.

        :param serial_number: The serial_number of this AssetIdentification.
        :type: str
        """

        self._serial_number = serial_number.strip() if serial_number is not None else serial_number

    @property
    def vendor_name(self):
        """Gets the vendor_name of this AssetIdentification.

        Vendor name of the asset.

        :return: The vendor_name of this AssetIdentification.
        :rtype: str
        """
        return self._vendor_name

    @vendor_name.setter
    def vendor_name(self, vendor_name):
        """Sets the vendor_name of this AssetIdentification.

        Vendor name of the asset.

        :param vendor_name: The vendor_name of this AssetIdentification.
        :type: str
        """

        self._vendor_name = vendor_name.strip() if vendor_name is not None else vendor_name

    @property
    def vendor_number(self):
        """Gets the vendor_number of this AssetIdentification.

        Vendor number of the asset.

        :return: The vendor_number of this AssetIdentification.
        :rtype: int
        """
        return self._vendor_number

    @vendor_number.setter
    def vendor_number(self, vendor_number):
        """Sets the vendor_number of this AssetIdentification.

        Vendor number of the asset.

        :param vendor_number: The vendor_number of this AssetIdentification.
        :type: int
        """

        self._vendor_number = vendor_number

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.property_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if the identification is equivalent"""
        if not isinstance(other, AssetIdentification):
            return False

        stripped_serial_number = self.serial_number.lstrip("0") if self.serial_number is not None else None
        stripped_other_serial_number = other.serial_number.lstrip("0") if other.serial_number is not None else None
        if stripped_serial_number != stripped_other_serial_number:
            return False

        model_number_valid = self._valid_identification_number(self.model_number)
        other_model_number_valid = self._valid_identification_number(other.model_number)
        if model_number_valid or other_model_number_valid:
            if self.model_number != other.model_number:
                return False
        elif self.model_name != other.model_name:
            return False

        vendor_number_valid = self._valid_identification_number(self.vendor_number)
        other_vendor_number_valid = self._valid_identification_number(other.vendor_number)
        vendor_name = self.vendor_name if self.vendor_name is not None else ''
        other_vendor_name = other.vendor_name if other.vendor_name is not None else ''
        if vendor_number_valid or other_vendor_number_valid:
            if self.vendor_number != other.vendor_number:
                return False
        elif vendor_name != other_vendor_name:
            return False

        return True

    @staticmethod
    def _valid_identification_number(number):
        return number is not None and number != 0

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self.__eq__(other)

    def __hash__(self) -> int:
        stripped_serial_number = self.serial_number.lstrip("0") if self.serial_number is not None else None
        return hash((self.model_name or '',
                     self.model_number,
                     stripped_serial_number,
                     self.vendor_name or '',
                     self.vendor_number))
