# coding: utf-8
"""
    Model to be associated with assets stored in the merged cache.
"""

import pprint

from asset_discovery.models.reporting_status import ReportingStatus


class AssetMetadata:
    property_types = {
        'data_sources': 'list[str]',
        'last_updated_timestamp': 'datetime',
        'version': 'int'
    }

    serialization_keys = {
        'data_sources': 'dataSources',
        'last_updated_timestamp': 'lastUpdatedTimestamp',
        'version': 'version'
    }

    """
    Model to be associated with assets stored in the merged cache.

    Summary of relevant metadata:
    - data_sources: a list of plugin ids that report that asset
    - reporting_status: a status based on how many (or none) plugins report an asset
    - last_updated_timestamp: timestamp updated if the asset changes on the merged cache
    - version: version updated if the asset changes on the merged cache
    """
    def __init__(self, data_sources=None, last_updated_timestamp=None, asset_identification=None, version=None):
        self._data_sources = data_sources
        self._last_updated_timestamp = last_updated_timestamp
        self._asset_identification = asset_identification
        self._version = version

    @property
    def reporting_status(self):
        if not self._data_sources:
            return ReportingStatus.UNREPORTED_RESOURCE
        if len(self._data_sources) == 1:
            return ReportingStatus.SINGLE_SOURCE_RESOURCE
        return ReportingStatus.MULTI_SOURCE_RESOURCE

    @property
    def data_sources(self):
        return self._data_sources

    @data_sources.setter
    def data_sources(self, value):
        self._data_sources = value

    @property
    def last_updated_timestamp(self):
        return self._last_updated_timestamp

    @last_updated_timestamp.setter
    def last_updated_timestamp(self, value):
        self._last_updated_timestamp = value

    @property
    def asset_identification(self):
        return self._asset_identification

    @asset_identification.setter
    def asset_identification(self, value):
        self._asset_identification = value

    @property
    def version(self):
        return self._version

    @version.setter
    def version(self, value):
        self._version = value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.property_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AssetMetadata):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self.__eq__(other)
