"""
    Model representing an asset update.
"""

import pprint

from asset_discovery.models.asset_identification import AssetIdentification
from asset_discovery.models.defaults import Defaults

DEFAULT = Defaults.DEFAULT_VALUE


class AssetUpdate:
    """
    Model representing an asset update.
    """
    property_types = {
        'model_name': 'str',
        'model_number': 'int',
        'serial_number': 'str',
        'vendor_name': 'str',
        'vendor_number': 'int',
        'bus_type': 'BusType',
        'provided_bus_type': 'BusType',
        'name': 'str',
        'asset_type': 'AssetType',
        'firmware_version': 'str',
        'hardware_version': 'str',
        'visa_resource_name': 'str',
        'temperature_sensors': 'list[TemperatureSensor]',
        'health': 'AssetHealthData',
        'supports_self_calibration': 'bool',
        'supports_external_calibration': 'bool',
        'supports_self_test': 'bool',
        'supports_reset': 'bool',
        'self_calibration': 'SelfCalibration',
        'is_ni_asset': 'bool',
        'location': 'LocationUpdate',
        'external_calibration': 'ExternalCalibration',
        'ip_addresses': 'List[str]',
        'hostname': 'str',
        'properties': 'dict(str, str)',
        'keywords': 'list[str]',
    }

    serialization_keys = {
        'model_name': 'modelName',
        'model_number': 'modelNumber',
        'serial_number': 'serialNumber',
        'vendor_name': 'vendorName',
        'vendor_number': 'vendorNumber',
        'bus_type': 'busType',
        'provided_bus_type': 'providedBusType',
        'name': 'name',
        'asset_type': 'assetType',
        'firmware_version': 'firmwareVersion',
        'hardware_version': 'hardwareVersion',
        'visa_resource_name': 'visaResourceName',
        'temperature_sensors': 'temperatureSensors',
        'health': 'health',
        'supports_self_calibration': 'supportsSelfCalibration',
        'supports_external_calibration': 'supportsExternalCalibration',
        'supports_self_test': 'supportsSelfTest',
        'supports_reset': 'supportsReset',
        'self_calibration': 'selfCalibration',
        'is_ni_asset': 'isNIAsset',
        'location': 'location',
        'external_calibration': 'externalCalibration',
        'ip_addresses': 'ipAddresses',
        'hostname': 'hostname',
        'properties': 'properties',
        'keywords': 'keywords',
    }

    def __init__(self,
                 model_name=DEFAULT,
                 model_number=DEFAULT,
                 serial_number=DEFAULT,
                 vendor_name=DEFAULT,
                 vendor_number=DEFAULT,
                 bus_type=DEFAULT,
                 provided_bus_type=DEFAULT,
                 name=DEFAULT,
                 asset_type=DEFAULT,
                 firmware_version=DEFAULT,
                 hardware_version=DEFAULT,
                 visa_resource_name=DEFAULT,
                 temperature_sensors=DEFAULT,
                 health=DEFAULT,
                 supports_self_calibration=DEFAULT,
                 supports_external_calibration=DEFAULT,
                 supports_self_test=DEFAULT,
                 supports_reset=DEFAULT,
                 self_calibration=DEFAULT,
                 is_ni_asset=DEFAULT,
                 location=DEFAULT,
                 external_calibration=DEFAULT,
                 ip_addresses=DEFAULT,
                 properties=DEFAULT,
                 hostname=DEFAULT,
                 keywords=DEFAULT):
        self._model_name = DEFAULT
        self._model_number = DEFAULT
        self._serial_number = DEFAULT
        self._vendor_name = DEFAULT
        self._vendor_number = DEFAULT
        self._bus_type = DEFAULT
        self._provided_bus_type = DEFAULT
        self._name = DEFAULT
        self._asset_type = DEFAULT
        self._firmware_version = DEFAULT
        self._hardware_version = DEFAULT
        self._visa_resource_name = DEFAULT
        self._temperature_sensors = DEFAULT
        self._health = DEFAULT
        self._supports_self_calibration = DEFAULT
        self._supports_external_calibration = DEFAULT
        self._supports_self_test = DEFAULT
        self._supports_reset = DEFAULT
        self._self_calibration = DEFAULT
        self._is_ni_asset = DEFAULT
        self._location = DEFAULT
        self._external_calibration = DEFAULT
        self._ip_addresses = DEFAULT
        self._hostname = DEFAULT
        self._properties = DEFAULT
        self._keywords = DEFAULT

        if model_name is not DEFAULT:
            self.model_name = model_name
        if model_number is not DEFAULT:
            self.model_number = model_number
        if serial_number is not DEFAULT:
            self.serial_number = serial_number
        if vendor_name is not DEFAULT:
            self.vendor_name = vendor_name
        if vendor_number is not DEFAULT:
            self.vendor_number = vendor_number
        if bus_type is not DEFAULT:
            self.bus_type = bus_type
        if provided_bus_type is not DEFAULT:
            self.provided_bus_type = provided_bus_type
        if name is not DEFAULT:
            self.name = name
        if asset_type is not DEFAULT:
            self.asset_type = asset_type
        if firmware_version is not DEFAULT:
            self.firmware_version = firmware_version
        if hardware_version is not DEFAULT:
            self.hardware_version = hardware_version
        if visa_resource_name is not DEFAULT:
            self.visa_resource_name = visa_resource_name
        if temperature_sensors is not DEFAULT:
            self.temperature_sensors = temperature_sensors
        if health is not DEFAULT:
            self.health = health
        if supports_self_calibration is not DEFAULT:
            self.supports_self_calibration = supports_self_calibration
        if supports_external_calibration is not DEFAULT:
            self.supports_external_calibration = supports_external_calibration
        if supports_self_test is not DEFAULT:
            self.supports_self_test = supports_self_test
        if supports_reset is not DEFAULT:
            self.supports_reset = supports_reset
        if self_calibration is not DEFAULT:
            self.self_calibration = self_calibration
        if is_ni_asset is not DEFAULT:
            self.is_ni_asset = is_ni_asset
        if location is not DEFAULT:
            self.location = location
        if external_calibration is not DEFAULT:
            self.external_calibration = external_calibration
        if ip_addresses is not DEFAULT:
            self.ip_addresses = ip_addresses
        if hostname is not DEFAULT:
            self.hostname = hostname
        if properties is not DEFAULT:
            self.properties = properties
        if keywords is not DEFAULT:
            self.keywords = keywords

    @staticmethod
    def create_from_identification(asset_identification):
        return AssetUpdate(
            model_name=asset_identification.model_name,
            model_number=asset_identification.model_number,
            serial_number=asset_identification.serial_number,
            vendor_name=asset_identification.vendor_name,
            vendor_number=asset_identification.vendor_number
        )

    @property
    def model_name(self):
        """Gets the model_name of this AssetUpdate.

        Model name of the asset.

        :return: The model_name of this AssetUpdate.
        :rtype: str
        """
        return self._model_name

    @model_name.setter
    def model_name(self, model_name):
        """Sets the model_name of this AssetUpdate.

        Model name of the asset.

        :param model_name: The model_name of this AssetUpdate.
        :type: str
        """

        self._model_name = model_name.strip() if model_name is not None else model_name

    @property
    def model_number(self):
        """Gets the model_number of this AssetUpdate.

        Model number of the asset.

        :return: The model_number of this AssetUpdate.
        :rtype: int
        """
        return self._model_number

    @model_number.setter
    def model_number(self, model_number):
        """Sets the model_number of this AssetUpdate.

        Model number of the asset.

        :param model_number: The model_number of this AssetUpdate.
        :type: int
        """

        self._model_number = model_number

    @property
    def serial_number(self):
        """Gets the serial_number of this AssetUpdate.

        Serial number of the asset.

        :return: The serial_number of this AssetUpdate.
        :rtype: str
        """
        return self._serial_number

    @serial_number.setter
    def serial_number(self, serial_number):
        """Sets the serial_number of this AssetUpdate.

        Serial number of the asset.

        :param serial_number: The serial_number of this AssetUpdate.
        :type: str
        """

        self._serial_number = serial_number.strip() if serial_number is not None else serial_number

    @property
    def vendor_name(self):
        """Gets the vendor_name of this AssetUpdate.

        Vendor name of the asset.

        :return: The vendor_name of this AssetUpdate.
        :rtype: str
        """
        return self._vendor_name

    @vendor_name.setter
    def vendor_name(self, vendor_name):
        """Sets the vendor_name of this AssetUpdate.

        Vendor name of the asset.

        :param vendor_name: The vendor_name of this AssetUpdate.
        :type: str
        """

        self._vendor_name = vendor_name.strip() if vendor_name is not None else vendor_name

    @property
    def vendor_number(self):
        """Gets the vendor_number of this AssetUpdate.

        Vendor number of the asset.

        :return: The vendor_number of this AssetUpdate.
        :rtype: int
        """
        return self._vendor_number

    @vendor_number.setter
    def vendor_number(self, vendor_number):
        """Sets the vendor_number of this AssetUpdate.

        Vendor number of the asset.

        :param vendor_number: The vendor_number of this AssetUpdate.
        :type: int
        """

        self._vendor_number = vendor_number

    @property
    def bus_type(self):
        """Gets the bus_type of this AssetUpdate.


        :return: The bus_type of this AssetUpdate.
        :rtype: BusType
        """
        return self._bus_type

    @bus_type.setter
    def bus_type(self, bus_type):
        """Sets the bus_type of this AssetUpdate.


        :param bus_type: The bus_type of this AssetUpdate.
        :type: BusType
        """

        self._bus_type = bus_type

    @property
    def provided_bus_type(self):
        """Gets the provided_bus_type of this AssetUpdate.


        :return: The provided_bus_type of this AssetUpdate.
        :rtype: BusType
        """
        return self._provided_bus_type

    @provided_bus_type.setter
    def provided_bus_type(self, provided_bus_type):
        """Sets the provided_bus_type of this AssetUpdate.


        :param provided_bus_type: The provided_bus_type of this AssetUpdate.
        :type: BusType
        """

        self._provided_bus_type = provided_bus_type

    @property
    def name(self):
        """Gets the name of this AssetUpdate.

        Name of the asset.

        :return: The name of this AssetUpdate.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this AssetUpdate.

        Name of the asset.

        :param name: The name of this AssetUpdate.
        :type: str
        """

        self._name = name

    @property
    def asset_type(self):
        """Gets the asset_type of this AssetUpdate.


        :return: The asset_type of this AssetUpdate.
        :rtype: AssetType
        """
        return self._asset_type

    @asset_type.setter
    def asset_type(self, asset_type):
        """Sets the asset_type of this AssetUpdate.


        :param asset_type: The asset_type of this AssetUpdate.
        :type: AssetType
        """

        self._asset_type = asset_type

    @property
    def firmware_version(self):
        """Gets the firmware_version of this AssetUpdate.

        Firmware version of the asset.

        :return: The firmware_version of this AssetUpdate.
        :rtype: str
        """
        return self._firmware_version

    @firmware_version.setter
    def firmware_version(self, firmware_version):
        """Sets the firmware_version of this AssetUpdate.

        Firmware version of the asset.

        :param firmware_version: The firmware_version of this AssetUpdate.
        :type: str
        """

        self._firmware_version = firmware_version

    @property
    def hardware_version(self):
        """Gets the hardware_version of this AssetUpdate.

        Hardware version of the asset.

        :return: The hardware_version of this AssetUpdate.
        :rtype: str
        """
        return self._hardware_version

    @hardware_version.setter
    def hardware_version(self, hardware_version):
        """Sets the hardware_version of this AssetUpdate.

        Hardware version of the asset.

        :param hardware_version: The hardware_version of this AssetUpdate.
        :type: str
        """

        self._hardware_version = hardware_version

    @property
    def visa_resource_name(self):
        """Gets the visa_resource_name of this AssetUpdate.

        VISA resource name of the asset.

        :return: The visa_resource_name of this AssetUpdate.
        :rtype: str
        """
        return self._visa_resource_name

    @visa_resource_name.setter
    def visa_resource_name(self, visa_resource_name):
        """Sets the visa_resource_name of this AssetUpdate.

        VISA resource name of the asset.

        :param visa_resource_name: The visa_resource_name of this AssetUpdate.
        :type: str
        """

        self._visa_resource_name = visa_resource_name

    @property
    def temperature_sensors(self):
        """Gets the temperature_sensors of this AssetUpdate.

        An array of temperature sensor information.

        :return: The temperature_sensors of this AssetUpdate.
        :rtype: list[TemperatureSensor]
        """
        return self._temperature_sensors

    @temperature_sensors.setter
    def temperature_sensors(self, temperature_sensors):
        """Sets the temperature_sensors of this AssetUpdate.

        An array of temperature sensor information.

        :param temperature_sensors: The temperature_sensors of this AssetUpdate.
        :type: list[TemperatureSensor]
        """

        self._temperature_sensors = temperature_sensors

    @property
    def health(self):
        """Gets the health of this AssetUpdate.

        An object representing health information.

        :return: The health information of this AssetUpdate.
        :rtype: AssetHealthData
        """
        return self._health

    @health.setter
    def health(self, health):
        """Sets the health information of this AssetUpdate.

        An object representing health information.

        :param health: The health information of this AssetUpdate.
        :type: AssetHealthData
        """

        self._health = health

    @property
    def supports_self_calibration(self):
        """Gets the supports_self_calibration of this AssetUpdate.

        Whether the asset supports self-calibration.

        :return: The supports_self_calibration of this AssetUpdate.
        :rtype: bool
        """
        return self._supports_self_calibration

    @supports_self_calibration.setter
    def supports_self_calibration(self, supports_self_calibration):
        """Sets the supports_self_calibration of this AssetUpdate.

        Whether the asset supports self-calibration.

        :param supports_self_calibration: The supports_self_calibration of this AssetUpdate.
        :type: bool
        """

        self._supports_self_calibration = supports_self_calibration

    @property
    def supports_external_calibration(self):
        """Gets the supports_external_calibration of this AssetUpdate.

        Whether the asset supports external calibration.

        :return: The supports_external_calibration of this AssetUpdate.
        :rtype: bool
        """
        return self._supports_external_calibration

    @supports_external_calibration.setter
    def supports_external_calibration(self, supports_external_calibration):
        """Sets the supports_external_calibration of this AssetUpdate.

        Whether the asset supports external calibration.

        :param supports_external_calibration: The supports_external_calibration of this AssetUpdate.
        :type: bool
        """

        self._supports_external_calibration = supports_external_calibration

    @property
    def supports_self_test(self):
        """Gets the supports_self_test of this AssetUpdate.

        Whether the asset supports self-test

        :return: The supports_self_test of this AssetUpdate.
        :rtype: bool
        """
        return self._supports_self_test

    @supports_self_test.setter
    def supports_self_test(self, supports_self_test):
        """Sets the supports_self_test of this AssetUpdate.

        Whether the asset supports self-test

        :param supports_self_test: The supports_self_test of this AssetUpdate.
        :type: bool
        """

        self._supports_self_test = supports_self_test

    @property
    def supports_reset(self):
        """Gets the supports_reset of this AssetUpdate.

        Whether the asset supports reset

        :return: The supports_reset of this AssetUpdate.
        :rtype: bool
        """
        return self._supports_reset

    @supports_reset.setter
    def supports_reset(self, supports_reset):
        """Sets the supports_reset of this AssetUpdate.

        Whether the asset supports reset

        :param supports_reset: The supports_reset of this AssetUpdate.
        :type: bool
        """

        self._supports_reset = supports_reset

    @property
    def self_calibration(self):
        """Gets the self_calibration of this AssetUpdate.


        :return: The self_calibration of this AssetUpdate.
        :rtype: SelfCalibration
        """
        return self._self_calibration

    @self_calibration.setter
    def self_calibration(self, self_calibration):
        """Sets the self_calibration of this AssetUpdate.


        :param self_calibration: The self_calibration of this AssetUpdate.
        :type: SelfCalibration
        """

        self._self_calibration = self_calibration

    @property
    def is_ni_asset(self):
        """Gets the is_ni_asset of this AssetUpdate.

        Whether this asset is an NI asset (true) or a third-party asset (false).

        :return: The is_ni_asset of this AssetUpdate.
        :rtype: bool
        """
        return self._is_ni_asset

    @is_ni_asset.setter
    def is_ni_asset(self, is_ni_asset):
        """Sets the is_ni_asset of this AssetUpdate.

        Whether this asset is an NI asset (true) or a third-party asset (false).

        :param is_ni_asset: The is_ni_asset of this AssetUpdate.
        :type: bool
        """

        self._is_ni_asset = is_ni_asset

    @property
    def location(self):
        """Gets the location of this AssetUpdate.


        :return: The location of this AssetUpdate.
        :rtype: LocationUpdate
        """
        return self._location

    @location.setter
    def location(self, location):
        """Sets the location of this AssetUpdate.


        :param location: The location of this AssetUpdate.
        :type: LocationUpdate
        """

        self._location = location

    @property
    def external_calibration(self):
        """Gets the external_calibration of this AssetUpdate.


        :return: The external_calibration of this AssetUpdate.
        :rtype: ExternalCalibration
        """
        return self._external_calibration

    @external_calibration.setter
    def external_calibration(self, external_calibration):
        """Sets the external_calibration of this AssetUpdate.


        :param external_calibration: The external_calibration of this AssetUpdate.
        :type: ExternalCalibration
        """

        self._external_calibration = external_calibration

    @property
    def ip_addresses(self):
        """Gets the ip_addresses of this AssetUpdate.

        :return: The ip_addresses of this AssetUpdate.
        :rtype: list(str)
        """
        return self._ip_addresses

    @ip_addresses.setter
    def ip_addresses(self, ip_addresses):
        """Sets the ip_addresses of this AssetUpdate.

        :param ip_addresses: The ip_addresses of this AssetUpdate.
        :type: list(str)
        """

        self._ip_addresses = ip_addresses

    @property
    def hostname(self):
        """Gets the hostname of this AssetUpdate.

        :return: The hostname of this AssetUpdate.
        :rtype: str
        """
        return self._hostname

    @hostname.setter
    def hostname(self, hostname):
        """Sets the hostname of this AssetUpdate.

        :param hostname: The properties of this AssetUpdate.
        :type: str
        """

        self._hostname = hostname

    @property
    def properties(self):
        """Gets the properties of this AssetUpdate.

        Key-value-pair metadata associated with an asset.

        :return: The properties of this AssetUpdate.
        :rtype: dict(str, str)
        """
        return self._properties

    @properties.setter
    def properties(self, properties):
        """Sets the properties of this AssetUpdate.

        Key-value-pair metadata associated with an asset.

        :param properties: The properties of this AssetUpdate.
        :type: dict(str, str)
        """

        self._properties = properties

    @property
    def keywords(self):
        """Gets the keywords of this AssetUpdate.

        Words or phrases associated with an asset.

        :return: The keywords of this AssetUpdate.
        :rtype: list[str]
        """
        return self._keywords

    @keywords.setter
    def keywords(self, keywords):
        """Sets the keywords of this AssetUpdate.

        Words or phrases associated with an asset.

        :param keywords: The keywords of this AssetUpdate.
        :type: list[str]
        """

        self._keywords = keywords

    @property
    def identification(self):
        return AssetIdentification(
            model_name=self.model_name,
            model_number=self.model_number,
            serial_number=self.serial_number,
            vendor_name=self.vendor_name,
            vendor_number=self.vendor_number
        )

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.property_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AssetUpdate):
            return False

        return (self.identification == other.identification and
                self.bus_type == other.bus_type and
                self.provided_bus_type == other.provided_bus_type and
                self.name == other.name and
                self.asset_type == other.asset_type and
                self.firmware_version == other.firmware_version and
                self.hardware_version == other.hardware_version and
                self.visa_resource_name == other.visa_resource_name and
                self.temperature_sensors == other.temperature_sensors and
                self.health == other.health and
                self.supports_self_calibration == other.supports_self_calibration and
                self.self_calibration == other.self_calibration and
                self.supports_external_calibration == other.supports_external_calibration and
                self.external_calibration == other.external_calibration and
                self.supports_self_test == other.supports_self_test and
                self.supports_reset == other.supports_reset and
                self.is_ni_asset == other.is_ni_asset and
                self.location == other.location and
                self.external_calibration == other.external_calibration and
                self.ip_addresses == other.ip_addresses and
                self.hostname == other.hostname and
                self.properties == other.properties and
                self.keywords == other.keywords)

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self.__eq__(other)
