# coding: utf-8

"""
    Model for a calibration operator.
"""

import pprint


class CalibrationOperator:
    """
    Model for a calibration operator.
    """

    """
    Attributes:
      property_types (dict): The key is attribute name
                            and the value is attribute type.
      serialization_keys (dict): The key is attribute name
                            and the value is json key in definition.
    """
    property_types = {
        'display_name': 'str',
        'user_id': 'str'
    }

    serialization_keys = {
        'display_name': 'displayName',
        'user_id': 'userId'
    }

    def __init__(self, display_name=None, user_id=None):
        self._display_name = None
        self._user_id = None

        if display_name is not None:
            self.display_name = display_name
        if user_id is not None:
            self.user_id = user_id

    @property
    def display_name(self):
        """Gets the display_name of this CalibrationOperator

        Display name of logged in user

        :return: The display_name of this CalibrationOperator
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """Sets the display_name of this CalibrationOperator.

        Display name of logged in user

        :param display_name: The display_name of this CalibrationOperator
        :type: str
        """

        self._display_name = display_name

    @property
    def user_id(self):
        """Gets the user_id of this CalibrationOperator

        Identifier of the logged in user

        :return: The user_id of this CalibrationOperator
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this CalibrationOperator.

        Identifier of the logged in user

        :param user_id: The user_id of this CalibrationOperator
        :type: str
        """

        self._user_id = user_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.property_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CalibrationOperator):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self.__eq__(other)
