import pprint

from datetime import datetime
from typing import List

from asset_discovery.models import FanSensorModel


class CoolingHealth:
    """
    Model representing the cooling system health information.
    """

    property_types = {
        'fan_mode': 'str',
        'supported_fan_modes': 'List[str]',
        'user_desired_fan_speed': 'int',
        'manual_rpm_upper_bound': 'int',
        'manual_rpm_lower_bound': 'int',
        'cooling_profile': 'str',
        'supported_cooling_profiles': 'List[str]',
        'cooling_operational_state': 'int',
        'power_supply_fans': 'List[FanSensorModel]',
        'chassis_fans': 'List[FanSensorModel]',
        'timestamp': 'datetime'
    }

    serialization_keys = {
        'fan_mode': 'fanMode',
        'supported_fan_modes': 'supportedFanModes',
        'user_desired_fan_speed': 'userDesiredFanSpeed',
        'manual_rpm_upper_bound': 'manualRpmUpperBound',
        'manual_rpm_lower_bound': 'manualRpmLowerBound',
        'cooling_profile': 'coolingProfile',
        'supported_cooling_profiles': 'supportedCoolingProfiles',
        'cooling_operational_state': 'coolingOperationalState',
        'power_supply_fans': 'powerSupplyFans',
        'chassis_fans': 'chassisFans',
        'timestamp': 'timestamp'
    }

    def __init__(self,
                 fan_mode: str = None,
                 supported_fan_modes: List[str] = None,
                 user_desired_fan_speed: int = None,
                 manual_rpm_upper_bound: int = None,
                 manual_rpm_lower_bound: int = None,
                 cooling_profile: str = None,
                 supported_cooling_profiles: List[str] = None,
                 cooling_operational_state: str = None,
                 power_supply_fans: List[FanSensorModel] = None,
                 chassis_fans: List[FanSensorModel] = None,
                 timestamp: datetime = None):
        self._fan_mode = fan_mode
        self._supported_fan_modes = supported_fan_modes
        self._user_desired_fan_speed = user_desired_fan_speed
        self._manual_rpm_upper_bound = manual_rpm_upper_bound
        self._manual_rpm_lower_bound = manual_rpm_lower_bound
        self._cooling_profile = cooling_profile
        self._supported_cooling_profiles = supported_cooling_profiles
        self._cooling_operational_state = cooling_operational_state
        self._power_supply_fans = power_supply_fans
        self._chassis_fans = chassis_fans
        self._timestamp = timestamp

    @property
    def fan_mode(self) -> str:
        return self._fan_mode

    @fan_mode.setter
    def fan_mode(self, fan_mode: str) -> None:
        self._fan_mode = fan_mode

    @property
    def supported_fan_modes(self) -> List[str]:
        return self._supported_fan_modes

    @supported_fan_modes.setter
    def supported_fan_modes(self, supported_fan_modes: List[str]) -> None:
        self._supported_fan_modes = supported_fan_modes

    @property
    def user_desired_fan_speed(self) -> int:
        return self._user_desired_fan_speed

    @user_desired_fan_speed.setter
    def user_desired_fan_speed(self, user_desired_fan_speed: int) -> None:
        self._user_desired_fan_speed = user_desired_fan_speed

    @property
    def manual_rpm_upper_bound(self) -> int:
        return self._manual_rpm_upper_bound

    @manual_rpm_upper_bound.setter
    def manual_rpm_upper_bound(self, manual_rpm_upper_bound: int) -> None:
        self._manual_rpm_upper_bound = manual_rpm_upper_bound

    @property
    def manual_rpm_lower_bound(self) -> int:
        return self._manual_rpm_lower_bound

    @manual_rpm_lower_bound.setter
    def manual_rpm_lower_bound(self, manual_rpm_lower_bound: int) -> None:
        self._manual_rpm_lower_bound = manual_rpm_lower_bound

    @property
    def cooling_profile(self) -> str:
        return self._cooling_profile

    @cooling_profile.setter
    def cooling_profile(self, cooling_profile: str) -> None:
        self._cooling_profile = cooling_profile

    @property
    def supported_cooling_profiles(self) -> List[str]:
        return self._supported_cooling_profiles

    @supported_cooling_profiles.setter
    def supported_cooling_profiles(self, supported_cooling_profiles: List[str]) -> None:
        self._supported_cooling_profiles = supported_cooling_profiles

    @property
    def cooling_operational_state(self) -> str:
        return self._cooling_operational_state

    @cooling_operational_state.setter
    def cooling_operational_state(self, cooling_operational_state: str) -> None:
        self._cooling_operational_state = cooling_operational_state

    @property
    def power_supply_fans(self) -> List[FanSensorModel]:
        return self._power_supply_fans

    @power_supply_fans.setter
    def power_supply_fans(self, power_supply_fans: List[FanSensorModel]) -> None:
        self._power_supply_fans = power_supply_fans

    @property
    def chassis_fans(self) -> List[FanSensorModel]:
        return self._chassis_fans

    @chassis_fans.setter
    def chassis_fans(self, chassis_fans: List[FanSensorModel]) -> None:
        self._chassis_fans = chassis_fans

    @property
    def timestamp(self) -> datetime:
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp: datetime) -> None:
        self._timestamp = timestamp

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in self.property_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CoolingHealth):
            return False

        return (self._fan_mode == other.fan_mode and
                self._supported_fan_modes == other.supported_fan_modes and
                self._user_desired_fan_speed == other.user_desired_fan_speed and
                self._manual_rpm_upper_bound == other.manual_rpm_upper_bound and
                self._manual_rpm_lower_bound == other.manual_rpm_lower_bound and
                self._cooling_profile == other.cooling_profile and
                self._supported_cooling_profiles == other.supported_cooling_profiles and
                self._cooling_operational_state == other.cooling_operational_state and
                self._power_supply_fans == other.power_supply_fans and
                self._chassis_fans == other.chassis_fans and
                self._timestamp == other.timestamp)

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self.__eq__(other)
