import pprint
from typing import Any, Dict


class FanSensorModel:
    """
    Model representing fan sensor data
    """

    property_types = {
        'name': 'str',
        'reading': 'Any',
        'state': 'Any',
        'properties': 'Dict[str, Any]'
    }

    serialization_keys = {
        'name': 'name',
        'reading': 'reading',
        'state': 'state',
        'properties': 'properties'
    }

    def __init__(self,
                 name: str = None,
                 reading: float = None,
                 state: int = None,
                 properties: Dict[str, Any] = None):
        self._name = name
        self._reading = reading
        self._state = state
        self._properties = properties

    @property
    def name(self) -> str:
        """Gets the name of this SensorModel

        Name of the sensor.

        :return: The name of this SensorModel.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: str) -> None:
        """Sets the name of this SensorModel.

        Name of the sensor.

        :param name: The name of this SensorModel.
        :type: str
        """
        self._name = name

    @property
    def reading(self) -> float:
        """Gets the reading of this SensorModel

        Reading of the sensor.

        :return: The reading of this SensorModel.
        :rtype: float
        """
        return self._reading

    @reading.setter
    def reading(self, reading: float) -> None:
        """Sets the reading of this SensorModel.

        Reading of the sensor.

        :param reading: The reading of this SensorModel.
        :type: float
        """
        self._reading = reading

    @property
    def state(self) -> int:
        """Gets the state of this SensorModel

        State of the sensor.

        :return: The state of this SensorModel.
        :rtype: int
        """
        return self._state

    @state.setter
    def state(self, state: int) -> None:
        """Sets the state of this SensorModel

        State of the sensor.

        :param state: The state of this sensor.
        :type: int
        """
        self._state = state

    @property
    def properties(self) -> Dict[str, Any]:
        """Gets the properties of this SensorModel

        Properties of the sensor.

        :return: The properties of this SensorModel.
        :rtype: dict[str, Any]
        """
        return self._properties

    @properties.setter
    def properties(self, properties: Dict[str, Any]) -> None:
        """Sets the properties of this SensorModel

        Properties of the sensor.

        :param properties: The properties of this SensorModel.
        :type: dict[str, Any]
        """
        self._properties = properties

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.property_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __hash__(self):
        return hash(str(self))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, FanSensorModel):
            return False

        return (self.name == other.name and
                self.reading == other.reading and
                self.state == other.state and
                self.properties == other.properties)

    def __ne__(self, other):
        return not self.__eq__(other)
