import pprint

from datetime import datetime

from asset_discovery.models.operational_state_value import OperationalStateValue


class OperationalState:

    property_types = {
        'state_number': 'int',
        'timestamp': 'datetime',
        'state': 'str',
        'status': 'str'
    }

    serialization_keys = {
        'state_number': 'stateNumber',
        'timestamp': 'timestamp',
        'state': 'state',
        'status': 'status'
    }

    operational_state_values = {
        0: OperationalStateValue.UNKNOWN,
        1: OperationalStateValue.NORMAL,
        256: OperationalStateValue.DEGRADED,
        512: OperationalStateValue.CONFIG_WARNING,
        1024: OperationalStateValue.PREDICTIVE_FAILURE,
        65536: OperationalStateValue.RECOVERABLE_ERROR_PROGRAMMATIC,
        131072: OperationalStateValue.RECOVERABLE_ERROR_USER_ACTION,
        262144: OperationalStateValue.NON_RECOVERABLE_ERROR
    }

    _normal_mask = 255
    _warning_mask = 65280
    _error_mask = 16711680

    def __init__(self,
                 state_number=None,
                 timestamp=None):
        self._state_number = state_number
        self._timestamp = timestamp

    @property
    def state_number(self) -> int:
        return self._state_number

    @state_number.setter
    def state_number(self, state_number) -> None:
        self._state_number = state_number

    @property
    def timestamp(self) -> datetime:
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp) -> None:
        self._timestamp = timestamp

    @property
    def state(self) -> str:
        return self.operational_state_values.get(self.state_number, '')

    @property
    def status(self) -> str:
        if (self._error_mask & self.state_number) != 0:
            return 'Error'
        if (self._warning_mask & self.state_number) != 0:
            return 'Warning'
        if (self._normal_mask & self.state_number) != 0:
            return 'Normal'
        return 'Unknown'

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.property_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OperationalState):
            return False

        return (self.state_number == other.state_number and
                self.timestamp == other.timestamp)

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self.__eq__(other)
