import pprint
from datetime import datetime
from typing import List, Any

from asset_discovery.models import SensorModel


class SensorCollection:
    property_types = {
        'timestamp': 'datetime',
        'sensors': 'List[SensorModel]'
    }

    serialization_keys = {
        'timestamp': 'timestamp',
        'sensors': 'sensors'
    }

    def __init__(self, sensors: List[SensorModel], timestamp: datetime):
        self._timestamp = timestamp
        self._sensors = sensors

    @property
    def sensors(self) -> List[SensorModel]:
        return self._sensors

    @sensors.setter
    def sensors(self, sensors) -> None:
        self._sensors = sensors

    @property
    def timestamp(self) -> datetime:
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp: datetime) -> None:
        self._timestamp = timestamp

    @staticmethod
    def has_any_readings(sensor_collection) -> bool:
        return (sensor_collection is not None and
                sensor_collection.sensors is not None and len(sensor_collection.sensors) > 0)

    def __len__(self):
        return len(self.sensors)

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.property_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, SensorCollection):
            return False

        return (self.timestamp == other.timestamp and
                self.sensors == other.sensors)

    def __ne__(self, other):
        return not self.__eq__(other)
