from asset_discovery.plugins.sysapi_imports.repository.models.attribute_tag import AttributeTag


class PropertyBagRequest:
    def __init__(self, resource_uri, properties=None):
        if properties is None:
            self._properties = []
        else:
            self._properties = properties
        self._resource_uri = resource_uri

    def set_standard_properties(self):
        # Properties that are commented, namely
        # DriverUserAlias0, NumberOfDrivers, DriverProgName0
        # are prohibited to be sent to populate_props.
        # Leaving them in here to have a complete list of tags
        # used to populate an asset model.
        self._properties = []
        self.add_standard_properties()

    def add_standard_properties(self):
        self._properties.extend([
            AttributeTag.ModelName,
            # ModelNumber
            AttributeTag.CSeriesModuleModelCode,
            AttributeTag.ModelNumber,
            #
            AttributeTag.SerialNumber,
            AttributeTag.VendorName,
            AttributeTag.VendorNumber,
            AttributeTag.ConnectsToBusType,
            # AttributeTag.DriverUserAlias0,
            # IsSystemController
            AttributeTag.ItemHasService,
            AttributeTag.NumberOfServices,
            AttributeTag.ServiceType0,
            #
            AttributeTag.FirmwareRevision,
            AttributeTag.HardwareRevision,
            # VisaResourceName
            # AttributeTag.NumberOfDrivers,
            # AttributeTag.DriverProgName0,
            AttributeTag.DriverItemName0,
            # TempSensors
            AttributeTag.SensorTempCount,
            AttributeTag.SensorTempReading0,
            AttributeTag.SensorTempName0,
            #
            AttributeTag.CalIntSupport,
            AttributeTag.CalExtSupport,
            # SelfCal
            AttributeTag.CalIntLastTime,
            AttributeTag.CalIntTempCount,
            AttributeTag.CalIntLastTempC,
            AttributeTag.CalIntTempName0,
            AttributeTag.CalIntLastLimited,
            #
            AttributeTag.IsProductNI,
            # Location
            # IsPresentStatus causes significant delays on network devices that are not reachable
            # BUG 1528668: https://ni.visualstudio.com/DevCentral/_workitems/edit/1528668
            # AttributeTag.IsPresentStatus,
            AttributeTag.ConnectsToLinkName,
            AttributeTag.ProvidesLinkName,
            AttributeTag.SlotNumber,
            # ExtCal
            AttributeTag.CalExtLastTime,
            AttributeTag.CalExtNextTime,
            AttributeTag.CalExtLastTempC,
            AttributeTag.CalExtLastLimited,
            AttributeTag.CalExtRecommendedInterval,
            AttributeTag.CalMiscComments,
            AttributeTag.CalExtChecksum
        ])

    def set_health_properties(self):
        self._properties = []
        self.add_health_properties()

    def add_health_properties(self):
        self._properties.extend([
            # TemperatureSensors
            AttributeTag.SensorTempCount,
            AttributeTag.SensorTempName0,
            AttributeTag.SensorTempReading0,
            AttributeTag.SensorTempLowerCritical0,
            AttributeTag.SensorTempUpperCritical0,
            AttributeTag.SensorTempHealthState0,
            AttributeTag.SensorTempLowerDamageRisk0,
            AttributeTag.SensorTempUpperDamageRisk0,
            AttributeTag.SensorTempLowerUser0,
            AttributeTag.SensorTempUpperUser0,
            AttributeTag.SensorTempAlarm0,
            AttributeTag.SensorTempAccuracy0,
            AttributeTag.SensorTempAlarmPollingInterval0,
            # VoltageSensors
            AttributeTag.SensorVoltCount,
            AttributeTag.SensorVoltName0,
            AttributeTag.SensorVoltReading0,
            AttributeTag.SensorVoltLowerCritical0,
            AttributeTag.SensorVoltUpperCritical0,
            AttributeTag.SensorVoltHealthState0,
            AttributeTag.SensorVoltLowerDamageRisk0,
            AttributeTag.SensorVoltUpperDamageRisk0,
            AttributeTag.SensorVoltLowerUser0,
            AttributeTag.SensorVoltUpperUser0,
            AttributeTag.SensorVoltAlarm0,
            AttributeTag.SensorVoltAccuracy0,
            AttributeTag.SensorVoltAlarmPollingInterval0,
            AttributeTag.SensorVoltNominal0,
            # PowerSensors
            AttributeTag.SensorPowerCount,
            AttributeTag.SensorPowerName0,
            AttributeTag.SensorPowerReading0,
            AttributeTag.SensorPowerUpperCritical0,
            AttributeTag.SensorPowerHealthState0,
            AttributeTag.SensorPowerUpperDamageRisk0,
            AttributeTag.SensorPowerUpperUser0,
            AttributeTag.SensorPowerAlarm0,
            AttributeTag.SensorPowerAccuracy0,
            AttributeTag.SensorPowerAlarmPollingInterval0,
            # CoolingSensors
            AttributeTag.SensorFanCount,
            AttributeTag.SensorFanName0,
            AttributeTag.SensorFanReading0,
            AttributeTag.SensorFanHealthState0,
            AttributeTag.PowerSupplyFanCount0,
            AttributeTag.PowerSupplyFanReading0,
            AttributeTag.FanMode,
            AttributeTag.SupportedFanModes,
            AttributeTag.FanUserRpm,
            AttributeTag.FanManualRpmLowerBound,
            AttributeTag.FanManualRpmUpperBound,
            AttributeTag.CoolingProfile,
            AttributeTag.SupportedCoolingProfiles,
            AttributeTag.CoolingOperationalState,
            # Lifetime power on hours
            AttributeTag.LifetimePowerOnHours,
            # Operational state
            AttributeTag.OperationalState,
        ])

    @property
    def resource_uri(self):
        return self._resource_uri

    @property
    def properties(self):
        return self._properties

    @properties.setter
    def properties(self, props):
        self._properties = props
