from asset_discovery.models.sensor_type import SensorType
from asset_discovery.plugins.sysapi_imports.repository.base_sensor_attributes import BaseSensorAttributes
from asset_discovery.plugins.sysapi_imports.repository.base_sensor_serializer import AbstractSensorSerializer
from asset_discovery.plugins.sysapi_imports.repository.models import AttributeTag


class PowerSensorSerializer(AbstractSensorSerializer):
    def __init__(self):
        super().__init__(numeric_attribute_tags=self._get_power_numeric_attribute_tags(),
                         base_name="Power",
                         sensor_type=SensorType.POWER)

    @staticmethod
    def _get_power_numeric_attribute_tags():
        return BaseSensorAttributes(
            count=AttributeTag.SensorPowerCount,
            name0=AttributeTag.SensorPowerName0,
            reading0=AttributeTag.SensorPowerReading0,
            upper_critical0=AttributeTag.SensorPowerUpperCritical0,
            health_state0=AttributeTag.SensorPowerHealthState0,
            upper_damage_risk0=AttributeTag.SensorPowerUpperDamageRisk0,
            upper_user0=AttributeTag.SensorPowerUpperUser0,
            alarm0=AttributeTag.SensorPowerAlarm0,
            accuracy0=AttributeTag.SensorPowerAccuracy0,
            polling_interval0=AttributeTag.SensorPowerAlarmPollingInterval0
        )

    def _on_sensor_parsing(self, i, bag, base_sensor):
        # power sensors do not have extra properties compared to the abstract sensors
        return base_sensor
