from asset_discovery.repository.path_resolver import PathResolver
from asset_discovery.repository.settings_provider import BaseSettingsProvider
from asset_discovery.plugins.sysapi_imports.environment import Environment


class SettingsProvider(BaseSettingsProvider):
    NI_INSTALLERS_REG_PATH = 'SOFTWARE\\National Instruments\\Common\\Installer'
    NI_INSTALLERS_REG_KEY_APP_DATA = 'NIPUBAPPDATADIR'

    DATA_DIRECTORY = 'Data'
    ASSETS_DIRECTORY = 'Assets'
    OVERWRITTEN_PROPS_DIRECTORY = 'OverwrittenSysApiProps'

    __instance = None

    @staticmethod
    def get_instance():
        """ Static access method. """
        if SettingsProvider.__instance is None:
            return SettingsProvider()
        return SettingsProvider.__instance

    def __init__(self):
        """ Virtually private constructor. """
        if SettingsProvider.__instance is not None:
            self.get_instance()
        else:
            SettingsProvider.__instance = self

    def get_store_directory_path(self):
        if Environment.DEBUG_MODE:
            return Environment.DEBUG_JSON_PATH
        return PathResolver.get_joined_skyline_appdata_dir(
            SettingsProvider.DATA_DIRECTORY,
            SettingsProvider.ASSETS_DIRECTORY,
            SettingsProvider.OVERWRITTEN_PROPS_DIRECTORY)
