from asset_discovery.models.sensor_properties import SensorProperties
from asset_discovery.models.sensor_type import SensorType
from asset_discovery.plugins.sysapi_imports.repository.base_sensor_attributes import BaseSensorAttributes
from asset_discovery.plugins.sysapi_imports.repository.base_sensor_serializer import AbstractSensorSerializer
from asset_discovery.plugins.sysapi_imports.repository.models import AttributeTag


class TemperatureSensorSerializer(AbstractSensorSerializer):
    def __init__(self):
        super().__init__(numeric_attribute_tags=self._get_temperature_numeric_attribute_tags(),
                         base_name="Temp",
                         sensor_type=SensorType.TEMPERATURE)

    @staticmethod
    def _get_temperature_numeric_attribute_tags():
        return BaseSensorAttributes(
            count=AttributeTag.SensorTempCount,
            name0=AttributeTag.SensorTempName0,
            reading0=AttributeTag.SensorTempReading0,
            upper_critical0=AttributeTag.SensorTempUpperCritical0,
            health_state0=AttributeTag.SensorTempHealthState0,
            upper_damage_risk0=AttributeTag.SensorTempUpperDamageRisk0,
            upper_user0=AttributeTag.SensorTempUpperUser0,
            alarm0=AttributeTag.SensorTempAlarm0,
            accuracy0=AttributeTag.SensorTempAccuracy0,
            polling_interval0=AttributeTag.SensorTempAlarmPollingInterval0
        )

    def _on_sensor_parsing(self, i, bag, base_sensor):
        lower_critical = bag.get_property_or_default(AttributeTag.SensorTempLowerCritical0 + i, None)
        lower_damage_risk = bag.get_property_or_default(AttributeTag.SensorTempLowerDamageRisk0 + i, None)
        lower_user = bag.get_property_or_default(AttributeTag.SensorTempLowerUser0 + i, None)

        if lower_critical is not None:
            base_sensor.properties[SensorProperties.LowerCritical] = lower_critical
        if lower_damage_risk is not None:
            base_sensor.properties[SensorProperties.LowerRisk] = lower_damage_risk
        if lower_user is not None:
            base_sensor.properties[SensorProperties.LowerUser] = lower_user

        return base_sensor
