from collections import namedtuple

AssetWithUri = namedtuple('AssetWithUri', 'asset uri')


class InMemoryEntryMap:
    def __init__(self):
        self._identification_to_asset = {}
        self._uri_to_identifications = {}

    def get_all(self):
        result = [asset_with_uri.asset for asset_with_uri in list(self._identification_to_asset.values())]
        return result

    def get_identification_to_entry(self):
        return {k: v.asset for k, v in self._identification_to_asset.items()}

    def exists(self, asset_identification):
        return self.get_by_identification(asset_identification) is not None

    def get_by_identification(self, asset_identification):
        asset_with_uri = self._identification_to_asset.get(asset_identification, None)
        return asset_with_uri.asset if asset_with_uri is not None else None

    def get_by_uri(self, uri):
        asset_identifications = self._uri_to_identifications.get(uri, None)
        if asset_identifications is None:
            return []
        return [self.get_by_identification(identification) for identification in asset_identifications]

    def get_uri(self, asset_identification):   # TODO: Introduce a DTO for the namedtuple to not query this 2 times
        asset_with_uri = self._identification_to_asset.get(asset_identification, None)
        return asset_with_uri.uri if asset_with_uri is not None else None

    def add(self, uri, asset):
        self._identification_to_asset[asset.identification] = AssetWithUri(asset, uri)

        existing_identifications = self._uri_to_identifications.get(uri, None)
        if existing_identifications is None:
            self._uri_to_identifications[uri] = []
        self._uri_to_identifications[uri].append(asset.identification)

    def flush(self):
        self._identification_to_asset = {}
        self._uri_to_identifications = {}
